/*
 * write your k-means code in this file.  when executed, it should
 * read in a PPM image and and write a PGM image where each pixel's
 * intensity value is its cluster label.  you might want to read in
 * other data, like the number of clusters, and you might want to 
 * output some extra information about each cluster (for example,
 * its mean color).  you could also try transforming the input image
 * to a different color space before running k-means.
 */

#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>

using namespace std;

#define I(x,y,c)   (input_image[3*((y)*(width)+(x))+(c)])
#define O(x,y)     (output_image[(y)*(width)+(x)])


void write_pgm( FILE* f, int width, int height, unsigned char* data );
unsigned char* read_ppm( char* filename, int* width, int* height );
void write_pgm( FILE* f, int width, int height, unsigned char* data );
void mat_pgm(FILE* f,int** mat,int width, int height);
float*** ppm_mat( char* filename, int& width, int& height );
int** imatrix(int row, int col);


int main( int argc, char **argv )
{
     int width, height;
	 FILE* f;
	 float*** image_input;
	 int** label;

	 image_input=ppm_mat(argv[1],width,height);     //read image to a float image
	 label=imatrix(height,width);
	 for (int i=0;i<height;i++)
		 for (int j=0;j<width;j++)
			 label[i][j]=0;



	 // write your code here.  you may also add functions
	 // outside of main if you wish.

    
     mat_pgm(stdout,label,width,height);//write int matrix to image
	 return 0;
}



/*****************************************************************************
 *      Read pgm image into float matrix.
 *****************************************************************************/
float*** ppm_mat( char* filename, int& width, int& height )
{
	 int i,j,k;
	 FILE* f;
	 unsigned char* input_image;
	 float*** input_image_float;

	 input_image=read_ppm(filename,&width,&height);
	 input_image_float=new float** [height];
	 for (i=0;i<height;i++)
	 {
		 input_image_float[i]=new float* [width];
		 for (j=0;j<width;j++)
		 {
		    input_image_float[i][j]=new float[3];
			input_image_float[i][j][0]=(float)I(j,i,0);
			input_image_float[i][j][1]=(float)I(j,i,1);
			input_image_float[i][j][2]=(float)I(j,i,2);
		 }
	 }
	 return input_image_float;
}

/*****************************************************************************
 *      Write integer matrix to pgm image
 *****************************************************************************/
void mat_pgm(FILE* f,int** mat,int width, int height)
{
	 int i,j;
	 unsigned char* output;
	 output=new unsigned char [width*height];
	 for (i=0;i<width;i++)
		for (j=0;j<height;j++)
			output[j*width+i]=(unsigned char)mat[j][i];
    write_pgm( f, width, height, output );
}



unsigned char* read_ppm( char* filename, int* width, int* height )
{
     char buffer[80];
     int phase = 0;
     int type, maxval;
     int j;
     unsigned char* data;
     FILE* f;

     if ( strcmp( filename, "-" ) == 0 )
     {
	  f = stdin;
     }
     else
     {
	  f = fopen( filename, "rb" );
	  if ( f == NULL )
	       return NULL;
     }
     
     while ( phase < 4 )
     {
	  fgets( buffer, 80, f );
	  if ( buffer[0] == '#' ) continue;
	  switch( phase )
	  {
	     case 0: j = sscanf( buffer, "P%d %d %d %d\n", &type, width, height, &maxval ); break;
	     case 1: j = sscanf( buffer, "%d %d %d\n", width, height, &maxval ); break;
	     case 2: j = sscanf( buffer, "%d %d\n", height, &maxval ); break;
	     case 3: j = sscanf( buffer, "%d\n", &maxval ); break;
	     case 4: j = 0; break;
	  }
	  phase += j;
     }

     if ( type != 6 )
     {
	  fprintf( stderr, "bad input image format.\n" );
	  return NULL;
     }

     data = (unsigned char*)malloc ( *width * *height * 3 );
     fread( data, *width * *height * 3, 1, f );

     fclose( f );
     
     return data;
}



void write_pgm( FILE* f, int width, int height, unsigned char* data )
{
     fprintf( f, "P5\n%d %d\n%d\n", width, height, 255 );
     fwrite( data, width*height, 1, f );
     fclose( f );
}


/*****************************************************************************
 *      Allocate memory for integer matrix.
 *****************************************************************************/

int** imatrix(int row, int col)
{
   int i;
   int **m;

   m=new int* [row];
   for (i=0;i<row;i++)
   {
      m[i] = new int[col];
   }
   return m;
}